package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.numberrule;

import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.api.iservice.busi.INumberRulePackAttributeStrategyService;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerPart;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerPartRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.Calendar;
import java.util.Date;

/**
 * 武汉客户条码
 */
@Component
public class WuHanCustSnNumberRuleStrategyService implements INumberRulePackAttributeStrategyService {

    @Autowired
    private MesCustomerPartRepository customerPartRepository;

    @Override
    public GenSerialNoModel execute(GenSerialNoModel genSerialNoModel) {
        if (null == genSerialNoModel) {
            MesException.throwBusiException("入参缺少[GenSerialNoModel]!");
        }
        if (StringUtils.isEmpty(genSerialNoModel.getOrganizeCode())) {
            MesException.throwBusiException("入参缺少[organizeCode]!");
        }
        if (StringUtils.isEmpty(genSerialNoModel.getPartNo()) && StringUtils.isEmpty(genSerialNoModel.getCustPartNo())) {
            MesException.throwBusiException("入参缺少[custPartNo]!");
        }

        if (StringUtils.isEmpty(genSerialNoModel.getCustPartNo())) {
            genSerialNoModel.setCustPartNo(getMesCustomerPart(genSerialNoModel.getOrganizeCode(), genSerialNoModel.getPartNo()));
        }

        Calendar custCalendar = Calendar.getInstance();
        custCalendar.setTime(new Date());
        String prefix = genSerialNoModel.getCustPartNo().substring(0, genSerialNoModel.getCustPartNo().length() - 4);
        String partNo4 = genSerialNoModel.getCustPartNo().substring(genSerialNoModel.getCustPartNo().length() - 4).toUpperCase();
        genSerialNoModel.setPartNo(prefix + partNo4);
        genSerialNoModel.setCustPartNo(prefix + partNo4);
        String year = custCalendar.get(Calendar.YEAR)+ MesExtConstWords.EMPTY;
        genSerialNoModel.setYear(year.substring(2));
        genSerialNoModel.setDay(String.format("%03d",custCalendar.get(Calendar.DAY_OF_YEAR)));

        return genSerialNoModel;
    }

    private String getMesCustomerPart(String orgaizeCode, String partNo) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(orgaizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, MesExtConstWords.ERP_PART_NO, ddlPackBean);
        MesCustomerPart customerPart = customerPartRepository.getByProperty(ddlPackBean);
        if (null != customerPart) return customerPart.getCustPartNo();
        return partNo;
    }

    @Override
    public String transform(GenSerialNoModel genSerialNoModel, String serialNo) {
        // [)>%RS%06%GS%Y,4110000000000X,%GS%P,%GS%12V,545238347,%GS%T1A,A2B4C,%RS%%EOT%
        //截取替换为ascii码
        String[] split = serialNo.split("%RS%");
        StringBuilder rsStr = new StringBuilder();
        for (int i = 0; i < split.length-1; i++) {
            rsStr.append(split[i]).append((char) 30);
        }
        rsStr.append(split[split.length - 1]);
        StringBuilder gsStr = new StringBuilder();
        String[] split1 = rsStr.toString().split("%GS%");
        for (int i = 0; i < split1.length-1; i++) {
            gsStr.append(split1[i]).append((char) 29);
        }
        gsStr.append(split1[split1.length-1]);
        String[] split2 = gsStr.toString().split("%EOT%");
        return split2[0]+(char)4;
    }
}

/**
 * RS 为记录分隔符ASCII:30
 * GS 为分组符ASCII:29
 * EOT 为传输结束符ASCII:04
 * {splitRule} [)>%RS%06%GS%Y
 * {splitRule} vpps= 4110000000000X
 * {splitRule} %GS%P
 * {partNo} 8位客户零件号
 * {splitRule} %GS%12V
 * {splitRule} duns 545238347
 * {splitRule} %GS%T1A
 * {yydayofyear}
 * {splitRule} A2B4C
 * {serialNo} 4位 步长1
 * {splitRule}%RS%%EOT%
 *{spiltRule}{spiltRule}{spiltRule}{partNo}{spiltRule}{spiltRule}{spiltRule}{year}{day}{spiltRule}{serialNo}{spiltRule}
 * [)>%RS%06%GS%Y,4110000000000X,%GS%P,%GS%12V,545238347,%GS%T1A,A2B4C,%RS%%EOT%
 *  通用二维码组成部分('RS'为记录分隔符ASCII:30，'GS'为分组符ASCII:29,'EOT'为传输结束符ASCII:04)
 *  [)>+'RS'+06+'GS'+Y+vpps+'GS'+P+8位客户零件号+'GS'+12V+DUNS+'GS'+T+1位line num+1位班次号+2位年份+3位天数+1位追溯(精确追溯A，批次追溯@)+2工程更改记录+2位自定义+4位流水号+'RS'+'EOT'
 *  如：[)>+'RS'+06+'GS'+Y+4110000000000X+'GS'+P+12345678+'GS'+12V+545295227+'GS'+T+1A13304A2B4C0001+'RS'+'EOT'
 *
 *  vpps = Y00000000000000
 *  8位客户零件号=最后四位要大写
 *  Duns是什么？
 *  1位lineNumber是什么？ 固定值 1
 *  1位班次号是什么？
 *  2位年YY 24
 *  3位天-~365/366
 *  追溯
 *  A--精确追溯
 *  @--批次追溯
 *
 *  2位工程更改记录
 *  2位供应商自定义信息
 *  4位流水号，当班班次计数，每班班次归零
 *
 *  WH_CUST_SN_RULE
 */
